@@ -64,12 +64,14 @@ ags_automation_edit_drawing_area_button_press_event(GtkWidget *widget, GdkEventB
 
   void ags_automation_edit_drawing_area_button_press_position_cursor()
   {
+    double zoom_factor;
     gdouble c_range;
     guint g_range;
-    double zoom_factor;
+    gdouble value, step;
+    gdouble upper, lower, step_count;
 
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      c_range = exp(automation_edit->upper) - exp(automation_edit->lower);
+      c_range = (gdouble) AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT;
     }else{
       c_range = automation_edit->upper - automation_edit->lower;
     }
@@ -83,7 +85,14 @@ ags_automation_edit_drawing_area_button_press_event(GtkWidget *widget, GdkEventB
     automation_edit->cursor_position_x = (guint) (zoom_factor * (event->x + GTK_RANGE(automation_edit->hscrollbar)->adjustment->value)) / automation_edit->control_width;
     
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      automation_edit->cursor_position_y = log(((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) / g_range) * c_range);
+      lower = automation_edit->lower;
+      upper = automation_edit->upper;
+
+      step_count = AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT + 1.0;
+
+      step = (gdouble) GTK_WIDGET(automation_edit->drawing_area)->allocation.height - (gdouble) event->y;
+      
+      automation_edit->cursor_position_y = lower * pow(upper / lower, step / (step_count - 1));
     }else{
       automation_edit->cursor_position_y = (((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) / g_range) * c_range);
     }
@@ -96,14 +105,16 @@ ags_automation_edit_drawing_area_button_press_event(GtkWidget *widget, GdkEventB
   {
     AgsAcceleration *acceleration;
 
+    double zoom_factor;
     gdouble c_range;
     guint g_range;
-    double zoom_factor;
+    gdouble value, step;
+    gdouble upper, lower, step_count;
     
     acceleration = ags_acceleration_new();
 
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      c_range = exp(automation_edit->upper) - exp(automation_edit->lower);
+      c_range = (gdouble) AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT;
     }else{
       c_range = automation_edit->upper - automation_edit->lower;
     }
@@ -117,7 +128,14 @@ ags_automation_edit_drawing_area_button_press_event(GtkWidget *widget, GdkEventB
     acceleration->x = (guint) (zoom_factor * (event->x + GTK_RANGE(automation_edit->hscrollbar)->adjustment->value));
     
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      acceleration->y = log(((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) / g_range) * c_range);
+      lower = automation_edit->lower;
+      upper = automation_edit->upper;
+
+      step_count = AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT + 1.0;
+
+      step = (gdouble) GTK_WIDGET(automation_edit->drawing_area)->allocation.height - (gdouble) event->y;
+
+      acceleration->y = lower * pow(upper / lower, step / (step_count - 1));
     }else{
       acceleration->y = (((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) / g_range) * c_range);
     }
@@ -195,12 +213,14 @@ ags_automation_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEven
 
   void ags_automation_edit_drawing_area_button_release_position_cursor()
   {
+    double zoom_factor;
     gdouble c_range;
     guint g_range;
-    double zoom_factor;
+    gdouble value, step;
+    gdouble upper, lower, step_count;
 
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      c_range = exp(automation_edit->upper) - exp(automation_edit->lower);
+      c_range = (gdouble) AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT;
     }else{
       c_range = automation_edit->upper - automation_edit->lower;
     }
@@ -214,7 +234,13 @@ ags_automation_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEven
     automation_edit->cursor_position_x = (guint) (zoom_factor * (event->x + GTK_RANGE(automation_edit->hscrollbar)->adjustment->value)) / automation_edit->control_width;
     
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      automation_edit->cursor_position_y = log(((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) / g_range) * c_range);
+      lower = automation_edit->lower;
+      upper = automation_edit->upper;
+
+      step_count = AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT + 1.0;
+
+      step = (gdouble) GTK_WIDGET(automation_edit->drawing_area)->allocation.height - (gdouble) event->y;
+      automation_edit->cursor_position_y = lower * pow(upper / lower, step / (step_count - 1));
     }else{
       automation_edit->cursor_position_y = (((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) / g_range) * c_range);
     }
@@ -227,9 +253,11 @@ ags_automation_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEven
   {
     AgsAcceleration *acceleration;
     
+    double zoom_factor;
     gdouble c_range;
     guint g_range;
-    double zoom_factor;
+    gdouble value, step;
+    gdouble upper, lower, step_count;
     guint new_x;
     
     acceleration = automation_edit->current_acceleration;
@@ -239,7 +267,7 @@ ags_automation_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEven
     }
 
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      c_range = exp(automation_edit->upper) - exp(automation_edit->lower);
+      c_range = (gdouble) AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT;
     }else{
       c_range = automation_edit->upper - automation_edit->lower;
     }
@@ -253,7 +281,14 @@ ags_automation_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEven
     acceleration->x = (guint) (zoom_factor * (event->x + GTK_RANGE(automation_edit->hscrollbar)->adjustment->value));
     
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      acceleration->y = log(((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) / g_range) * c_range);
+      lower = automation_edit->lower;
+      upper = automation_edit->upper;
+
+      step_count = AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT + 1.0;
+
+      step = (gdouble) GTK_WIDGET(automation_edit->drawing_area)->allocation.height - (gdouble) event->y;
+
+      acceleration->y = lower * pow(upper / lower, step / (step_count - 1));
     }else{
       acceleration->y = (((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) / g_range) * c_range);
     }
@@ -272,14 +307,16 @@ ags_automation_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEven
   
   void ags_automation_edit_drawing_area_button_release_delete_acceleration()
   {
+    double zoom_factor;
     gdouble c_range;
     guint g_range;
-    double zoom_factor;
+    gdouble value, step;
+    gdouble upper, lower, step_count;
     guint x;
     gdouble y;
 
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      c_range = exp(automation_edit->upper) - exp(automation_edit->lower);
+      c_range = (gdouble) AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT;
     }else{
       c_range = automation_edit->upper - automation_edit->lower;
     }
@@ -293,7 +330,13 @@ ags_automation_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEven
     x = (guint) zoom_factor * ((event->x + GTK_RANGE(automation_edit->hscrollbar)->adjustment->value));
     
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      y = log(((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) / g_range) * c_range);
+      lower = automation_edit->lower;
+      upper = automation_edit->upper;
+
+      step_count = AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT + 1.0;
+
+      step = (gdouble) GTK_WIDGET(automation_edit->drawing_area)->allocation.height - (gdouble) event->y;
+      y = lower * pow(upper / lower, step / (step_count - 1));
     }else{
       y = (((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) / g_range) * c_range);
     }
@@ -305,14 +348,16 @@ ags_automation_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEven
   
   void ags_automation_edit_drawing_area_button_release_select_acceleration()
   {
+    double zoom_factor;
     gdouble c_range;
     guint g_range;
-    double zoom_factor;
+    gdouble value, step;
+    gdouble upper, lower, step_count;
     guint x0, x1;
     gdouble y0, y1;
     
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      c_range = exp(automation_edit->upper) - exp(automation_edit->lower);
+      c_range = (gdouble) AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT;
     }else{
       c_range = automation_edit->upper - automation_edit->lower;
     }
@@ -326,7 +371,14 @@ ags_automation_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEven
     x0 = (guint) zoom_factor * automation_edit->selection_x0;
 
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      y0 = log((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - automation_edit->selection_y0) / g_range) * c_range;
+      lower = automation_edit->lower;
+      upper = automation_edit->upper;
+
+      step_count = AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT + 1.0;
+
+      step = (gdouble) GTK_WIDGET(automation_edit->drawing_area)->allocation.height - (gdouble) automation_edit->selection_y0;
+      
+      y0 = lower * pow(upper / lower, step / (step_count - 1));
     }else{
       y0 = ((gdouble) (GTK_WIDGET(automation_edit->drawing_area)->allocation.height - automation_edit->selection_y0) / g_range) * c_range;
     }
@@ -334,7 +386,14 @@ ags_automation_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEven
     x1 = (guint) zoom_factor * (event->x + GTK_RANGE(automation_edit->hscrollbar)->adjustment->value);
     
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      y1 = log(((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) + GTK_RANGE(automation_edit->vscrollbar)->adjustment->value) / g_range) * c_range;
+      lower = automation_edit->lower;
+      upper = automation_edit->upper;
+
+      step_count = AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT + 1.0;
+
+      step = (gdouble) GTK_WIDGET(automation_edit->drawing_area)->allocation.height - (gdouble) event->y;
+      
+      y1 = lower * pow(upper / lower, step / (step_count - 1));
     }else{
       y1 = (((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) + GTK_RANGE(automation_edit->vscrollbar)->adjustment->value) / g_range) * c_range;
     }
@@ -389,12 +448,14 @@ ags_automation_edit_drawing_area_motion_notify_event(GtkWidget *widget, GdkEvent
 
   void ags_automation_edit_drawing_area_motion_notify_position_cursor()
   {
+    double zoom_factor;
     gdouble c_range;
     guint g_range;
-    double zoom_factor;
+    gdouble value, step;
+    gdouble upper, lower, step_count;
 
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      c_range = exp(automation_edit->upper) - exp(automation_edit->lower);
+      c_range = (gdouble) AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT;
     }else{
       c_range = automation_edit->upper - automation_edit->lower;
     }
@@ -408,7 +469,13 @@ ags_automation_edit_drawing_area_motion_notify_event(GtkWidget *widget, GdkEvent
     automation_edit->cursor_position_x = (guint) zoom_factor * ((event->x + GTK_RANGE(automation_edit->hscrollbar)->adjustment->value));
 
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      automation_edit->cursor_position_y = log(((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) / g_range) * c_range);
+      lower = automation_edit->lower;
+      upper = automation_edit->upper;
+
+      step_count = AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT + 1.0;
+
+      step = (gdouble) GTK_WIDGET(automation_edit->drawing_area)->allocation.height - (gdouble) event->y;
+      automation_edit->cursor_position_y = lower * pow(upper / lower, step / (step_count - 1));
     }else{
       automation_edit->cursor_position_y = (((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) / g_range) * c_range);
     }
@@ -425,9 +492,11 @@ ags_automation_edit_drawing_area_motion_notify_event(GtkWidget *widget, GdkEvent
   {
     AgsAcceleration *acceleration;
     
+    double zoom_factor;
     gdouble c_range;
     guint g_range;
-    double zoom_factor;
+    gdouble value, step;
+    gdouble upper, lower, step_count;
     
     acceleration = automation_edit->current_acceleration;
     
@@ -436,7 +505,7 @@ ags_automation_edit_drawing_area_motion_notify_event(GtkWidget *widget, GdkEvent
     }
     
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      c_range = exp(automation_edit->upper) - exp(automation_edit->lower);
+      c_range = (gdouble) AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT;
     }else{
       c_range = automation_edit->upper - automation_edit->lower;
     }
@@ -450,7 +519,14 @@ ags_automation_edit_drawing_area_motion_notify_event(GtkWidget *widget, GdkEvent
     acceleration->x = (guint) zoom_factor * ((event->x + GTK_RANGE(automation_edit->hscrollbar)->adjustment->value));
     
     if((AGS_AUTOMATION_EDIT_LOGARITHMIC & (automation_edit->flags)) != 0){
-      acceleration->y = log(((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) / g_range) * c_range);
+      lower = automation_edit->lower;
+      upper = automation_edit->upper;
+
+      step_count = AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT + 1.0;
+
+      step = (gdouble) GTK_WIDGET(automation_edit->drawing_area)->allocation.height - (gdouble) event->y;
+
+      acceleration->y = lower * pow(upper / lower, step / (step_count - 1));
     }else{
       acceleration->y = (((GTK_WIDGET(automation_edit->drawing_area)->allocation.height - event->y) / g_range) * c_range);
     }
